library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
library work;
use work.all;

entity hw1A is
     port(
        key : in std_logic_vector(3 downto 0);   -- pushbutton switches
        sw : in std_logic_vector(9 downto 0);    -- slide switches
        ledg : out std_logic_vector(7 downto 0); -- green LEDs
		ledr : out std_logic_vector(9 downto 0); -- red LEDs
		hex0 : out std_logic_vector(6 downto 0); -- 7-segment display, digit 0
		hex1 : out std_logic_vector(6 downto 0);
		hex2 : out std_logic_vector(6 downto 0);
		hex3 : out std_logic_vector(6 downto 0)
		);
end hw1A;


architecture behavioural of hw1A is
	signal clk : std_logic;
	signal reset : std_logic;
	
	signal accum  : std_logic_vector(3 downto 0);
	signal plusone : std_logic_vector(3 downto 0);
	
	signal A_Q, B_Q : std_logic_vector(3 downto 0);
	signal add_out, and_out, sub_out, mux_out : std_logic_vector(3 downto 0);
	signal mul_out : std_logic_vector(7 downto 0);
	
begin

	-- Buttons to signals names so I don't get confused
	clk <= NOT key(0);
	reset <= NOT key(3);
	
	-- Two simple gates
	ledr(9) <= NOT( sw(9) OR sw(8) );
	ledr(8) <= NOT( sw(9) XOR sw(8) );

	-- Accumulator output to LEDs
	ledg(3 downto 0) <= plusone;
	h0: hex port map( i=> plusone, o => hex0);
	-- An accumulator, counts 1 every clock cycle
	plusone <= accum + 1;
	process (clk, reset) begin
		if(reset = '1') then
			accum <= "0000";
		elsif(clk'event and clk='1') then
			accum <= plusone;
		end if;
	end process;		

	-- Mini-datapath output to LEDs
	ledr(3 downto 0) <= B_Q;
	ledr(7 downto 4) <= A_Q;
	ledg(7 downto 4) <= mux_out;
	
	h3: hex port map ( i => A_Q, o=>hex3);
	h2: hex port map ( i => B_Q, o=>hex2);
	h1: hex port map ( i => mux_out, o=>hex1);
	
	-- Outputs of the adder and other functions
	add_out <= A_Q + B_Q;
	sub_out <= A_Q - B_Q;
	mul_out <= A_Q * B_Q;
	and_out <= A_Q AND B_Q;
	
	-- Output mux to decide which ALU operation is on the LEDs
	with sw(1 downto 0) select
		mux_out <=  add_out when "00",
					sub_out when "01",
					mul_out(3 downto 0) when "10",
					and_out when others;
					
	-- The registers
	process (clk, reset) begin
		if(reset = '1') then
			A_Q <= "0000";
			B_Q <= "0000";
		elsif(clk'event and clk='1') then
			if(sw(2) = '1') then
				B_Q <= sw(7 downto 4);
			end if;
			if(sw(3) = '1') then
				A_Q <= sw(7 downto 4);
			end if;
		end if;
	end process;		

end behavioural;

