
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

entity datapath is
     port(
		selReg : in std_logic_vector(1 downto 0);
		wrReg : in std_logic;
		wrA : in std_logic;
		wrB : in std_logic;
		ALUop: in std_logic_vector(1 downto 0);
		selBus : in std_logic_vector(1 downto 0);
		selLED : in std_logic;
		switch_input : in std_logic_vector(7 downto 0);
		data_out : out std_logic_vector(7 downto 0);
		alu_out : out std_logic_vector(7 downto 0);
		reg_out : out std_logic_vector(7 downto 0);
		cc_out : out std_logic_vector(5 downto 0);
		reset : in std_logic;
		clk : in std_logic
		);
end datapath ;


architecture behavioural of datapath is
	signal A_Q, B_Q, R, rf_out, data_bus, data_bus_in : std_logic_vector(7 downto 0);
	signal v_add, v_sub, c_add, b_sub, n_flag, z_flag : std_logic; 
	signal Sm, Rm, Dm : std_logic; 
	signal alu_mult_out : std_logic_vector(15 downto 0);
	type regarray is array(3 downto 0) of std_logic_vector(7 downto 0);
	signal rf: regarray;

begin
	-- data bus input ('0000' & SW[13:10]) on DE2, '000000000' on DE1
	data_bus_in <= switch_input; 
	
	-- register file
	process (reset, clk) begin
		if(reset = '1' ) then
				rf ( 0 ) <= "00000000";
				rf ( 1 ) <= "00001111";
				rf ( 2 ) <= "11110000";
				rf ( 3 ) <= "11111111";
		elsif(clk'event AND clk='1') then
			if(wrReg = '1') then
				rf ( conv_integer(selReg) ) <= data_bus;
			end if;
		end if;
	end process;
	rf_out <= rf ( conv_integer(selReg) );

	-- Two registers (A,B) that feed the ALU
	process (clk, reset) begin
		if(reset = '1') then
			A_Q <= "00000000";
			B_Q <= "00000000";
		elsif(clk'event and clk='1') then
			if(wrA = '1') then
				A_Q <= data_bus;
			end if;
			if(wrB = '1') then
				B_Q <= data_bus;
			end if;
		end if;
	end process;
				
	-- ALU
	alu_mult_out <= A_Q * B_Q;
	with ALUop select 
		R <= 	A_Q + B_Q when "00",
				A_Q - B_Q when "01",
				alu_mult_out(7 downto 0) when "10",
				A_Q AND B_Q when "11";
	
	-- ALU flags (condition codes)
	Sm <= B_Q(7);  -- (S)ource
	Dm <= A_Q(7);  -- (D)est
	Rm <= R(7);    -- (R)esult
	-- N - Negative: 1 if the ALU result is negative
	n_flag <= Rm;
	-- Vadd - Signed Add Overflow (D + S): 1 if ADD overflows, only set for ADD instructions
	v_add <= (Sm AND Dm AND NOT Rm) OR (NOT Sm AND NOT Dm AND Rm);
	-- Vsub - Signed Subtract Overflow (D - S): 1 if SUB overflows, only set for SUB
	v_sub <= (NOT Sm AND Dm AND NOT Rm) OR (Sm AND NOT Dm AND Rm);
	-- C - Carry: 1 if an ADD generates a carry out
	c_add <= (Sm AND Dm) OR (NOT Rm AND Dm) OR (Sm AND NOT Rm);
	-- B - Borrow: 1 if SUB generates a borrow
	b_sub <= (Sm AND NOT Dm) OR (Rm AND NOT Dm) OR (Sm AND Rm); 
	-- Z - Zero: 1 if the ALU result (R) is 0
	with R select z_flag <= '1' when "00000000", '0' when others;
					
	-- Flag output
--	cc_out( 0 ) <= c_add when "00", '0' when others;
--	cc_out( 1 ) <= b_sub when "01", '0' when others;
--	cc_out( 2 ) <= v_add when "00", '0' when others;
--	cc_out( 3 ) <= v_sub when "01", '0' when others;
	cc_out( 0 ) <= c_add;
	cc_out( 1 ) <= b_sub;
	cc_out( 2 ) <= v_add;
	cc_out( 3 ) <= v_sub;
	cc_out( 4 ) <= n_flag;
	cc_out( 5 ) <= z_flag;
	
	-- Data bus input mux
	with selBus select
		data_bus <=	data_bus_in when "00",
					"00000001" when "01",
					rf_out when "10",
					R when "11";
					
	-- Data out selector to the LEDs
--	with selLED select
	--	data_out <=	data_bus when '0',
		--			R when '1';
	data_out <= data_bus;
	alu_out <= R;
	reg_out <= rf_out;	
	
end behavioural;